% Read/Write StagYY data
%
% Read binary output files of STAG3D and write them into Paraview files
%
% Boris Kaus, 26.2.2008
%
% PJT edits August 2010:
% - spherical: move coordinate transform just before writing, otherwise reset
% - spherical: centre domain around equator, rotate velocity components
% - YinYang: transform velocity components
% - new Annulus mode (replaced previous 'Cylindrical' mode)

% fcrameri, 26.05.2011

% residual temperature not tested for other than Yin-Yang grids !!!!

clear

fname           =   'm4_3d';                      % filename

% directory       =   '/work/stagyy';
% directory       = '/Volumes/Earth-ONE/work/+P_dependent_rheology/+3_t-dependent_weak_crust/5_thin_crust';
% directory       = '/Volumes/Earth-THREE/YinYang';
% directory       = '/work/STAG_CONTINENTS';
% directory       = '/work/STAG_EARTH';
% directory       = '/Volumes/F2/output';
% directory       = '/Volumes/F4a/YinYang';
directory       = '/Volumes/F2/data/archer';
directory     =   '/work/melting_cmb/stagyy140411_cbl';


% saveDirectory   = '/Volumes/F2/VTUs/yy_lid1_bot/+add';
saveDirectory   = directory;

% GridType        =   'YinYang';         % choose one of these geometries
GridType        =   'Cartesian';
%GridType        =   'Spherical';
% GridType        =   'Annulus';


% Specify which fields we want to read/write in addition to T:
WriteResidualT  =   false;           %write residual temperature files
WriteViscosity  =   false;        	% Read & write viscosity files?
WriteVelocity   =   false;        	% Read & write velocity files?
WriteStress     =   false;
WriteStrainrate =   false;
WriteComposition=   false;
WriteMeltFrac   =   false;
WriteDefMech    =   false;


directory = strcat(directory,'/+op/',fname);
disp(['directory:      ',directory])

num = 1; % number of frame for name array

number_start = 104;  %0
number_end = 104;
for fname_number=number_start:2:number_end
  disp(['fname_number:   ',num2str(fname_number)])
  
    switch GridType
        case 'YinYang'

            % YING YANG GRID
            disp(['   Creating a YIN-YANG grid ...'])

            [nnb, ~, ~, ~,T_3D_1, T_3D_2, ~, rcmb]     = ReadStag3D2(directory, fname, fname_number, 'temperature');
            
            if nnb==-999
                % The file does not exist, and we have finished processing all
                % data

                % Create a PVD file, which contains all data including
                % time-information
                Create_PVD_file(FileNames,fname,saveDirectory);

                error('finished processing all files in directory')
            else
                
                if WriteResidualT
                    % Read temperature and remove horizontal mean
                    % temperature
                    [nnb, X_3D, Y_3D, Z_3D,T_3D_1, T_3D_2, time, rcmb]     = ReadStag3D2(directory, fname, fname_number, 'temperature');
                    Tres_3D_1 = zeros(size(T_3D_1));
                    Tres_3D_2 = zeros(size(T_3D_2));
                    for ilevel=1:size(T_3D_1,3)
                        Tres_3D_1(:,:,ilevel) = T_3D_1(:,:,ilevel) - mean(mean(T_3D_1(:,:,ilevel)));
                        Tres_3D_2(:,:,ilevel) = T_3D_2(:,:,ilevel) - mean(mean(T_3D_2(:,:,ilevel)));
                    end
                end
                
                if WriteVelocity
                    % Read pressure & velocity information
                    [~, ~, ~, ~,VX_3D_1, VY_3D_1, VZ_3D_1, P_3D_1, VX_3D_2, VY_3D_2, VZ_3D_2, P_3D_2, time] = ReadStag3D2(directory, fname, fname_number, 'velocity');
                end
                
                if WriteViscosity
                    % Read viscosity information
                    [nnb, X_3D, Y_3D, Z_3D,ETA_3D_1, ETA_3D_2, time, rcmb] = ReadStag3D2(directory, fname, fname_number, 'viscosity');
                end
                
                if WriteStress
                    % Read viscosity information
                    [nnb, X_3D, Y_3D, Z_3D,STR_3D_1, STR_3D_2, time, rcmb] = ReadStag3D2(directory, fname, fname_number, 'stress');
                end
                
                if WriteStrainrate
                    % Read viscosity information
                    [nnb, X_3D, Y_3D, Z_3D,ED_3D_1, ED_3D_2, time, rcmb] = ReadStag3D2(directory, fname, fname_number, 'strain rate');
                end
                
                if WriteComposition
                    % Read viscosity information
                    [nnb, X_3D, Y_3D, Z_3D,C_3D_1, C_3D_2, time, rcmb]     = ReadStag3D2(directory, fname, fname_number, 'composition');
                end
                
                if WriteMeltFrac
                    % Read viscosity information
                    [nnb, X_3D, Y_3D, Z_3D,F_3D_1, F_3D_2, time, rcmb]     = ReadStag3D2(directory, fname, fname_number, 'melt fraction');
                end
                
                if WriteDefMech
                    % Read deformation mechanism information
                    [nnb, X_3D, Y_3D, Z_3D,DEFM_3D_1, DEFM_3D_2, time, rcmb]     = ReadStag3D2(directory, fname, fname_number, 'defmech');
                end

                xs = X_3D(:,:,end)+pi/4;
                ys = Y_3D(:,:,end)-pi/4;
                zs = Z_3D(:,:,end);

                % Transform coordinates for Yin & Yang grids
                R  = Z_3D+rcmb;     lat = pi/4-X_3D;     lon = Y_3D-3*pi/4;
                
                X_3D_1 = R.*cos(lat).*cos(lon);   % Yin grid
                Y_3D_1 = R.*cos(lat).*sin(lon);
                Z_3D_1 = R.*sin(lat);

                X_3D_2 = -X_3D_1;       %  Yang grid
                Y_3D_2 =  Z_3D_1;
                Z_3D_2 =  Y_3D_1;
                
                % Transform velocities, if needed
                if WriteVelocity
                    Vtheta = VX_3D_1; Vphi = VY_3D_1; Vr = VZ_3D_1;                                 % on Yin grid
                    VX_3D_1 =  Vtheta.*sin(lat).*cos(lon) - Vphi.*sin(lon) + Vr.*cos(lat).*cos(lon);
                    VY_3D_1 =  Vtheta.*sin(lat).*sin(lon) + Vphi.*cos(lon) + Vr.*cos(lat).*sin(lon);
                    VZ_3D_1 = -Vtheta.*cos(lat)                            + Vr.*sin(lat);
                    Vtheta = VX_3D_2; Vphi = VY_3D_2; Vr = VZ_3D_2;                                 % on Yang grid
                    VX_3D_2 = -( Vtheta.*sin(lat).*cos(lon) - Vphi.*sin(lon) + Vr.*cos(lat).*cos(lon) );
                    VZ_3D_2 =  Vtheta.*sin(lat).*sin(lon) + Vphi.*cos(lon) + Vr.*cos(lat).*sin(lon);
                    VY_3D_2 = -Vtheta.*cos(lat)                            + Vr.*sin(lat);
                end

                % Write a ying-yang grid
                WriteStag3D_VTK_YinYang2;    % Write VTK XML file
                
                % Store the filename and time in a structure, which is later
                % used to create a pvd file, that contains all timesteps and
                % times of the files
                FileNames{num} = {time,fname_number,fname_vtk};

            end


        case 'Cartesian'
            % CARTESIAN GRID
            
            % Read temperature information
            [nnb, X_3D, Y_3D, Z_3D,T_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'temperature');

            if nnb==-999
                % The file does not exist, and we have finished processing all
                % data

                % Create a PVD file, which contains all data including
                % time-information
                Create_PVD_file(FileNames,fname,saveDirectory);

                error(['finished processing all files in directory'])

            else
                
                if WriteResidualT
                    [nnb, X_3D, Y_3D, Z_3D,T_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'temperature');
                    Tres_3D = zeros(size(T_3D));
                    for ilevel=1:size(T_3D,3)
                        Tres_3D(:,:,ilevel) = T_3D(:,:,ilevel) - mean(mean(T_3D(:,:,ilevel)));
                    end
                end
                
                if WriteVelocity
                    % Read pressure & velocity information
                    [~,~, ~, ~, VX_3D, VY_3D, VZ_3D, P_3D, time] = ReadStag3D2(directory, fname, fname_number, 'velocity');
                end

                if WriteViscosity
                    % Read viscosity information
                    [nnb,X_3D, Y_3D, Z_3D,ETA_3D, time]   = ReadStag3D2(directory, fname, fname_number, 'viscosity');
                end
                
                if WriteStress
                    % Read viscosity information
                    [nnb, X_3D, Y_3D, Z_3D,STR_3D, time] = ReadStag3D2(directory, fname, fname_number, 'stress');
                end
                
                if WriteStrainrate
                    % Read viscosity information
                    [nnb, X_3D, Y_3D, Z_3D,ED_3D, time] = ReadStag3D2(directory, fname, fname_number, 'strain rate');
                end
                
                if WriteComposition
                    % Read viscosity information
                    [nnb,X_3D, Y_3D, Z_3D,C_3D, time]   = ReadStag3D2(directory, fname, fname_number, 'composition');
                end

                if WriteMeltFrac
                    % Read viscosity information
                    [nnb,X_3D, Y_3D, Z_3D,F_3D, time]   = ReadStag3D2(directory, fname, fname_number, 'melt fraction');
                end
                
                if WriteDefMech
                    % Read deformation mechanism information
                    [nnb,X_3D, Y_3D, Z_3D,DEFM_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'defmech');
                end
                
                %WriteStag3D_LegacyVTK_Cartesian;   %-> Writes VTK legacy files
                WriteStag3D_VTK_Cartesian;          %-> Writes VTK-XML files
                
                % Store the filename and time in a structure, which is later
                % used to create a pvd file, that contains all timesteps and
                % times of the files
                FileNames{num} = {time,fname_number,fname_vtk};

            end
            
        case 'Spherical'
            % SPHERICAL GRID
            
            % Read temperature information
            [nnb, X_3D, Y_3D, Z_3D,T_3D, time, rcmb]     = ReadStag3D2(directory, fname, fname_number, 'temperature');

            if nnb==-999
                % The file does not exist, and we have finished processing all
                % data
                
                % Create a PVD file, which contains all data including
                % time-information
                Create_PVD_file(FileNames,fname,saveDirectory);
                error(['finished processing all files in directory'])

            else
                
                if WriteResidualT
                    [nnb, X_3D, Y_3D, Z_3D,T_3D, time, rcmb]     = ReadStag3D2(directory, fname, fname_number, 'temperature');
                    Tres_3D = zeros(size(T_3D));
                    for ilevel=1:size(T_3D,3)
                        Tres_3D(:,ilevel) = T_3D(:,ilevel) - mean(mean(T_3D(:,ilevel)));
                    end
                end
                
               if WriteVelocity
                    % Read pressure & velocity information
                    [~,~, ~, ~, VX_3D, VY_3D, VZ_3D, P_3D, time] = ReadStag3D2(directory, fname, fname_number, 'velocity');
                end

                if WriteViscosity
                    % Read viscosity information
                    [nnb,X_3D, Y_3D, Z_3D,ETA_3D, time]   = ReadStag3D2(directory, fname, fname_number, 'viscosity');
                end
                
                if WriteStress
                    % Read viscosity information
                    [nnb, X_3D, Y_3D, Z_3D,STR_3D, time] = ReadStag3D2(directory, fname, fname_number, 'stress');
                end
                
                if WriteStrainrate
                    % Read viscosity information
                    [nnb, X_3D, Y_3D, Z_3D,ED_3D, time] = ReadStag3D2(directory, fname, fname_number, 'strain rate');
                end

                if WriteComposition
                    % Read viscosity information
                    [nnb,X_3D, Y_3D, Z_3D,C_3D, time]   = ReadStag3D2(directory, fname, fname_number, 'composition');
                end

                if WriteMeltFrac
                    % Read viscosity information
                    [nnb,X_3D, Y_3D, Z_3D,F_3D, time]   = ReadStag3D2(directory, fname, fname_number, 'melt fraction');
                end
                
                if WriteDefMech
                    % Read deformation mechanism information
                    [nnb,X_3D, Y_3D, Z_3D,DEFM_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'defmech');
                end

                % Transform coordinates for Yin grid
                latmean = (max(max(max(X_3D))) + min(min(min(X_3D)))) / 2;       % spherical patch is centred at equator
                lonmean = (max(max(max(Y_3D))) + min(min(min(Y_3D)))) / 2;       
                R    = Z_3D + rcmb;  lat = latmean - X_3D;  lon = Y_3D - lonmean;
                X_3D = R.*cos(lat).*cos(lon);
                Y_3D = R.*cos(lat).*sin(lon);
                Z_3D = R.*sin(lat);
                
                % Transform velocities if they are needed
                if WriteVelocity
                    Vtheta = VX_3D; Vphi = VY_3D; Vr = VZ_3D;
                    VX_3D =  Vtheta.*sin(lat).*cos(lon) - Vphi.*sin(lon) + Vr.*cos(lat).*cos(lon);
                    VY_3D =  Vtheta.*sin(lat).*sin(lon) + Vphi.*cos(lon) + Vr.*cos(lat).*sin(lon);
                    VZ_3D = -Vtheta.*cos(lat)                            + Vr.*sin(lat);
                end

                %WriteStag3D_LegacyVTK_Cartesian;   %-> Writes VTK legacy files
                WriteStag3D_VTK_Cartesian;          %-> Writes VTK-XML files
                
                % Store the filename and time in a structure, which is later
                % used to create a pvd file, that contains all timesteps and
                % times of the files
                FileNames{num} = {time,fname_number,fname_vtk};

            end
            
        case 'Annulus'
            % SPHERICAL ANNULUS GRID
            % same as spherical but add extra points to avoid a gap in the
            % visualisation

            % Read temperature information
            [nnb, X_3D, Y_3D, Z_3D,T_3D, time, rcmb]     = ReadStag3D2(directory, fname, fname_number, 'temperature');
   
            
            % add one row for nicer visualization
            T_3D(1,end+1,:) = T_3D(1,1,:);

            if fname_number>number_end
                % The file does not exist, and we have finished processing all
                % data

                % Create a PVD file, which contains all data including
                % time-information
                Create_PVD_file(FileNames,fname,saveDirectory);
                
                error(['finished processing all files in directory'])

            else
                

                if WriteVelocity
                    % Read pressure & velocity information
                    [~,~, ~, ~, VX_3D, VY_3D, VZ_3D, P_3D, time] = ReadStag3D2(directory, fname, fname_number, 'velocity');
                    VX_3D(1,end+1,:) = VX_3D(1,1,:);
                    VY_3D(1,end+1,:) = VY_3D(1,1,:);
                    VZ_3D(1,end+1,:) = VZ_3D(1,1,:);
                    P_3D(1,end+1,:) =  P_3D(1,1,:);
                end
                
                if WriteResidualT
                    [nnb, X_3D, Y_3D, Z_3D,T_3D, time, rcmb]     = ReadStag3D2(directory, fname, fname_number, 'temperature');
                    Tres_3D = zeros(size(T_3D));
                    for ilevel=1:size(T_3D,3)
                        Tres_3D(:,ilevel) = T_3D(:,ilevel) - mean(mean(T_3D(:,ilevel)));
                    end
                end
               
                if WriteViscosity
                    % Read viscosity information
                    [nnb,X_3D, Y_3D, Z_3D,ETA_3D, time]   = ReadStag3D2(directory, fname, fname_number, 'viscosity');
                    ETA_3D(1,end+1,:) = ETA_3D(1,1,:);
                end

                if WriteStress
                    % Read viscosity information
                    [nnb, X_3D, Y_3D, Z_3D,STR_3D, time] = ReadStag3D2(directory, fname, fname_number, 'stress');
                    STR_3D(1,end+1,:) = STR_3D(1,1,:);
                end
                
                if WriteStrainrate
                    % Read viscosity information
                    [nnb, X_3D, Y_3D, Z_3D,ED_3D, time] = ReadStag3D2(directory, fname, fname_number, 'strain rate');
                    ED_3D(1,end+1,:) = ED_3D(1,1,:);
                end
                
                if WriteComposition
                    % Read viscosity information
                    [nnb,X_3D, Y_3D, Z_3D,C_3D, time]   = ReadStag3D2(directory, fname, fname_number, 'composition');
                    C_3D(1,end+1,:) = C_3D(1,1,:);
                end

                if WriteMeltFrac
                    % Read viscosity information
                    [nnb,X_3D, Y_3D, Z_3D,F_3D, time]   = ReadStag3D2(directory, fname, fname_number, 'melt fraction');
                    F_3D(1,end+1,:) = F_3D(1,1,:);
                end
                
                if WriteDefMech
                    % Read deformation mechanism information
                    [nnb,X_3D, Y_3D, Z_3D,DEFM_3D, time]     = ReadStag3D2(directory, fname, fname_number, 'defmech');
                    DEFM_3D(1,end+1,:) = DEFM_3D(1,1,:);
                end
                
                Y_3D(1,end+1,:) = Y_3D(1,1,:) + 2*pi;
                X_3D(1,end+1,:) = X_3D(1,1,:);
                Z_3D(1,end+1,:) = Z_3D(1,1,:);
                
                % Transform coordinates for Yin grid
                R    = Z_3D + rcmb;  lat = zeros(size(X_3D));  lon = Y_3D - pi;
                X_3D = R.*cos(lat).*cos(lon);
                Y_3D = R.*cos(lat).*sin(lon);
                Z_3D = R.*sin(lat);
                
                % Transform velocities if they are needed
                if WriteVelocity
                    Vtheta = VX_3D; Vphi = VY_3D; Vr = VZ_3D;
                    VX_3D =  Vtheta.*sin(lat).*cos(lon) - Vphi.*sin(lon) + Vr.*cos(lat).*cos(lon);
                    VY_3D =  Vtheta.*sin(lat).*sin(lon) + Vphi.*cos(lon) + Vr.*cos(lat).*sin(lon);
                    VZ_3D = -Vtheta.*cos(lat)                            + Vr.*sin(lat);
                end

                %WriteStag3D_LegacyVTK_Cartesian;   %-> Writes VTK legacy files
                WriteStag3D_VTK_Cartesian;          %-> Writes VTK-XML files
                
                % Store the filename and time in a structure, which is later
                % used to create a pvd file, that contains all timesteps and
                % times of the files
                FileNames{num} = {time,fname_number,fname_vtk};
            end

        otherwise

            error('Unknown gridtype')
    end

    num = num+1;

end

Create_PVD_file(FileNames,fname,saveDirectory);
disp('Finished')










